
"CaseControlDipSim" <-
  function(n=20,penetrance=c(0.01,0.05,0.9),sites=1010,var=100,freq=0.1,rho=0.1)
  {
    selectdip <- function(u,penetrance,n) {
      m <- rowSums(matrix(u,ncol=2,byrow=T))+1; 
      pu <- numeric(length(m));
      pu <- penetrance[m];
      sel <-matrix(runif(length(m))<pu,nrow=2,ncol=length(m),byrow=T)
      seli <- as.vector(sel)
      cat("got ",sum(seli)/2," affected individuals\n")
      if (sum(seli)<n)
        stop("need larger penetrance or more sporadic cases to get ",n);
      list(case=which(seli)[1:n],control=which(!seli)[1:n])
    }
    fmn <- freq-0.02
    f <- c((1-fmn)^2,2*fmn*(1-fmn),fmn^2)%*%penetrance;
    ss <- 2*round((n/2+4*sqrt(n/2))/f);
    cat("sample size is ", ss/2,", expectation of "
        ,round(ss*f/2)," affected individuals\nsampling tree\n")
    
    a <- mutate(simARG(ss,sites,rho),(var+1),paste("panel(",n,")"));
    f1 <- apply(a$d,2,mean)
 
    w <- which(abs(f1-freq)<0.02)
    if (length(w)==1)
   stop("no sites with the correct frequency\n");
    wh <- w[which(abs(w-var/2) == min(abs(w-var/2)))]
    pos <- a$position[wh]
    cat("chose locus ",wh,"with freq",f1[wh],"\n")
    u1 <- a$d[,wh]
    cc <- selectdip(u1,penetrance,n);
    a <- prune(a,c(cc$case,cc$control))
    ccdata <- list(graph=a,pos=pos,wh=wh,cc=cc
                   ,position=a$position[-wh]
                   ,d=rbind(a$d[cc$case,-wh],a$d[cc$control,-wh])
                   ,locations=rep(1:2,c(n,n))
                   ,SNP=c(a$d[cc$case,wh],a$d[cc$control,wh])
                   )
    class(ccdata) <-c("CCDip","sima")
    ccdata
  }

tree.CCDip <- function(a,wh,diffpos=0) {
  fixphylo <- function(phy,case,SNP)
  {
    lab <- as.numeric(phy$tip.label)+1
 #   print(lab)
    
    cs <- lab %in% case
    newlab <- lab
    newlab[cs] <- " *"
    newlab[!cs] <- " -"
    col <- rep("black",length(lab));
    if (!missing(SNP)) { 
      mutcase <- lab %in% SNP
      col[mutcase] <- "red"
    }
 #   print(newlab)
    phy$tip.label <- newlab;
    phy$col <- col;
    phy
  }
  mktable <- function(cc,S)
    {
      all <- c(cc$case,cc$control);
      #print(all)
      case <- all %in% cc$case
      SNP <- all %in% S 
      table(case,SNP)
     
    }

  if (missing(wh)) 
    posi <- a$pos+diffpos
  else posi <- a$position[wh]
  cat ("at position ",posi,"\n");
  if (posi>a$graph$sites) {
    warning("diffpos too large - using last site to the right")
    posi <- a$graph$sites
  }
  if (posi<1) {
    warning("diffpos too large (negative) - using last site to the left")
    posi <- 1
  }
  t1 <- tree(a$graph,posi)
  
  if (!missing(wh)) {
    samp <- c(a$cc$case,a$cc$control);
    SNP <- samp[a$d[,wh]==1]
    t1 <- fixphylo(t1,a$cc$case,SNP);
  }  else if (diffpos==0) {
    samp <- c(a$cc$case,a$cc$control);
    SNP <- samp[a$SNP==1]
    t1 <- fixphylo(t1,a$cc$case,SNP)  
  } else if (posi %in% a$positions) {
    whmut <- which(posi==positions)
    samp <- c(a$cc$case,a$cc$control);
    SNP <- samp[a$d[,whmut]==1]
    t1 <- fixphylo(t1,a$cc$case,SNP);
  } else {
    t1 <- fixphylo(t1,a$cc$case);  
  }
  if (!missing(SNP))
    tab <- mktable(a$cc,SNP);
  t1$table <- tab;
  class(t1) <- c("myphylo","phylo")
  t1;
}




 plot.myphylo <- 
function (x, type = "phylogram", use.edge.length = TRUE, node.pos = NULL, 
    show.node.label = FALSE, edge.color = "black", edge.width = 1, 
    font = 3, cex = par("cex"), adj = NULL, srt = 0, no.margin = FALSE, 
    root.edge = FALSE, label.offset = 0, underscore = FALSE, col="black",
    x.lim = NULL, y.lim = NULL, direction = "rightwards", lab4ut = "horizontal", 
    ...) 
{
    type <- match.arg(type, c("phylogram", "cladogram", "unrooted", 
        "radial"))
    direction <- match.arg(direction, c("rightwards", "leftwards", 
        "upwards", "downwards"))
    if (is.null(x$edge.length)) 
        use.edge.length <- FALSE
    tmp <- as.numeric(x$edge)
    nb.tip <- max(tmp)
    nb.node <- -min(tmp)
    if (type == "unrooted" || !use.edge.length) 
        root.edge <- FALSE
    if (type %in% c("phylogram", "cladogram")) {
        if (is.null(node.pos)) {
            if (type == "phylogram") 
                node.pos <- 1
            if (type == "cladogram") 
                node.pos <- if (!use.edge.length) 
                  2
                else 1
        }
        yy <- if (node.pos == 1) 
            node.height(x$edge)
        else node.height.clado(x$edge)
        if (!use.edge.length) {
            xx <- node.depth(x$edge) - 1
            xx <- max(xx) - xx
        }
        else {
            nms <- c(-(1:nb.node), 1:nb.tip)
            xx <- .C("node_depth_edgelength", as.integer(nb.tip), 
                as.integer(nb.node), as.integer(x$edge[, 1]), 
                as.integer(x$edge[, 2]), as.integer(nms), as.double(x$edge.length), 
                as.double(numeric(nb.tip + nb.node)), DUP = FALSE, 
                PACKAGE = "ape")[[7]]
            names(xx) <- as.character(nms)
        }
        if (root.edge) 
            switch(direction, rightwards = xx <- xx + x$root.edge, 
                leftwards = xx <- xx <- x$root.edge, upwards = yy <- yy + 
                  x$root.edge, downwards = yy <- yy - x$root.edge)
    }
    if (type == "unrooted") {
        XY <- if (use.edge.length) 
            unrooted.xy(nb.tip, nb.node, x$edge, x$edge.length)
        else unrooted.xy(nb.tip, nb.node, x$edge, rep(1, dim(x$edge)[1]))
        xx <- XY$M[, 1] - min(XY$M[, 1])
        yy <- XY$M[, 2] - min(XY$M[, 2])
    }
    if (type == "radial") {
        X <- node.depth(x$edge)
        X[X == 1] <- 0
        X <- 1 - X/nb.tip
        Y <- node.height(x$edge) * 2 * pi/nb.tip
        xx <- X * cos(Y)
        yy <- X * sin(Y)
    }
    if (type %in% c("phylogram", "cladogram") && direction != 
        "rightwards") {
        if (direction == "leftwards") {
            xx <- -xx
            xx <- xx - min(xx)
        }
        if (direction %in% c("upwards", "downwards")) {
            tmp <- yy
            yy <- xx
            xx <- tmp - min(tmp) + 1
            if (direction == "downwards") {
                yy <- -yy
                yy <- yy - min(yy)
            }
        }
    }
    if (no.margin) 
        par(mai = rep(0, 4))
    if (is.null(x.lim)) {
        if (type %in% c("phylogram", "cladogram")) {
            if (direction %in% c("rightwards", "leftwards")) {
                x.lim <- c(0, NA)
                x.lim[2] <- if (direction == "leftwards") 
                  max(xx["-1"] + nchar(x$tip.label) * 0.018 * 
                    max(xx) * cex)
                else max(xx[as.character(1:nb.tip)] + nchar(x$tip.label) * 
                  0.018 * max(xx) * cex)
            }
            else x.lim <- c(1, nb.tip)
        }
        if (type == "unrooted") {
            offset <- max(nchar(x$tip.label) * 0.018 * max(yy) * 
                cex)
            x.lim <- c(0 - offset, max(xx) + offset)
        }
        if (type == "radial") {
            offset <- max(nchar(x$tip.label) * 0.03 * cex)
            x.lim <- c(-1 - offset, 1 + offset)
        }
    }
    else {
        if (length(x.lim) == 1) {
            if (type %in% c("phylogram", "cladogram")) 
                x.lim <- if (direction %in% c("rightwards", "leftwards")) 
                  c(0, x.lim)
                else c(1, x.lim)
            if (type == "unrooted") {
                offset <- max(nchar(x$tip.label) * 0.018 * max(yy) * 
                  cex)
                x.lim <- c(0 - offset, x.lim)
            }
            if (type == "radial") {
                offset <- max(nchar(x$tip.label) * 0.03 * cex)
                x.lim <- c(-1 - offset, x.lim)
            }
        }
    }
    if (is.null(y.lim)) {
        if (type %in% c("phylogram", "cladogram")) {
            if (direction %in% c("upwards", "downwards")) {
                y.lim <- c(0, NA)
                y.lim[2] <- if (direction == "downwards") 
                  max(yy["-1"] + nchar(x$tip.label) * 0.018 * 
                    max(yy) * cex)
                else max(yy[as.character(1:nb.tip)] + nchar(x$tip.label) * 
                  0.018 * max(yy) * cex)
            }
            else y.lim <- c(1, nb.tip)
        }
        if (type == "unrooted") {
            offset <- max(nchar(x$tip.label) * 0.018 * max(yy) * 
                cex)
            y.lim <- c(0 - offset, max(yy) + offset)
        }
        if (type == "radial") {
            offset <- max(nchar(x$tip.label) * 0.03 * cex)
            y.lim <- c(-1 - offset, 1 + offset)
        }
    }
    else {
        if (length(y.lim) == 1) {
            if (type %in% c("phylogram", "cladogram")) 
                y.lim <- if (direction %in% c("rightwards", "leftwards")) 
                  c(1, y.lim)
                else c(0, y.lim)
            if (type == "unrooted") {
                offset <- max(nchar(x$tip.label) * 0.018 * max(yy) * 
                  cex)
                y.lim <- c(0 - offset, y.lim)
            }
            if (type == "radial") {
                offset <- max(nchar(x$tip.label) * 0.018 * max(yy) * 
                  cex)
                y.lim <- c(-1 - offset, y.lim)
            }
        }
    }
    edge.color <- rep(edge.color, length.out = dim(x$edge)[1])
    edge.width <- rep(edge.width, length.out = dim(x$edge)[1])
    plot(0, type = "n", xlim = x.lim, ylim = y.lim, xlab = "", 
        ylab = "", xaxt = "n", yaxt = "n", bty = "n", ...)
    if (is.null(adj)) 
        adj <- if (type %in% c("phylogram", "cladogram") && direction == 
            "leftwards") 
            1
        else 0
    if (type %in% c("phylogram", "cladogram")) {
        MAXSTRING <- max(strwidth(x$tip.label, cex = cex))
        if (direction == "rightwards") {
            lox <- label.offset + MAXSTRING * 1.05 * adj
            loy <- 0
        }
        if (direction == "leftwards") {
            lox <- -label.offset - MAXSTRING * 1.05 * (1 - adj)
            loy <- 0
            xx <- xx + MAXSTRING
        }
        if (direction %in% c("upwards", "downwards")) {
            psr <- par("usr")
            MAXSTRING <- MAXSTRING * 1.09 * (psr[4] - psr[3])/(psr[2] - 
                psr[1])
            loy <- label.offset + MAXSTRING * 1.05 * adj
            lox <- 0
            srt <- 90 + srt
            if (direction == "downwards") {
                loy <- -loy
                yy <- yy + MAXSTRING
                srt <- 180 + srt
            }
        }
    }
    if (type == "phylogram") 
        phylogram.plot(x$edge, nb.tip, nb.node, xx, yy, direction, 
            edge.color, edge.width)
    else cladogram.plot(x$edge, xx, yy, edge.color, edge.width)
    if (root.edge) 
        segments(0, yy["-1"], x$root.edge, yy["-1"])
    if (!underscore) 
        x$tip.label <- gsub("_", " ", x$tip.label)
    if (type %in% c("phylogram", "cladogram")) {
        text(xx[as.character(1:nb.tip)] + lox, yy[as.character(1:nb.tip)] + 
            loy, x$tip.label, adj = adj, font = font, srt = srt,col=x$col, 
            cex = cex)
    }
    if (type == "unrooted") {
        if (lab4ut == "horizontal") {
            y.adj <- x.adj <- numeric(nb.tip)
            sel <- abs(XY$axe) > 0.75 * pi
            x.adj[sel] <- -strwidth(x$tip.label)[sel] * 1.05
            sel <- abs(XY$axe) > pi/4 & abs(XY$axe) < 0.75 * 
                pi
            x.adj[sel] <- -strwidth(x$tip.label)[sel] * (2 * 
                abs(XY$axe)[sel]/pi - 0.5)
            sel <- XY$axe > pi/4 & XY$axe < 0.75 * pi
            y.adj[sel] <- strheight(x$tip.label)[sel]/2
            sel <- XY$axe < -pi/4 & XY$axe > -0.75 * pi
            y.adj[sel] <- -strheight(x$tip.label)[sel] * 0.75
            text(xx[as.character(1:nb.tip)] + x.adj, yy[as.character(1:nb.tip)] + 
                y.adj, x$tip.label, adj = c(adj, 0), font = font, 
                srt = srt, cex = cex,col=x$col)
        }
        else {
            adj <- as.numeric(abs(XY$axe) > pi/2)
            srt <- 180 * XY$axe/pi
            srt[as.logical(adj)] <- srt[as.logical(adj)] - 180
            for (i in 1:nb.tip) text(xx[as.character(i)], yy[as.character(i)], 
                cex = cex, x$tip.label[i], adj = adj[i], font = font, 
                srt = srt[i],col=x$col)
        }
    }
    if (type == "radial") {
        angle <- acos(xx[as.character(1:nb.tip)]) * 180/pi
        s1 <- angle > 90 & yy[as.character(1:nb.tip)] > 0
        s2 <- angle < 90 & yy[as.character(1:nb.tip)] < 0
        s3 <- angle > 90 & yy[as.character(1:nb.tip)] < 0
        angle[s1] <- angle[s1] + 180
        angle[s2] <- -angle[s2]
        angle[s3] <- 180 - angle[s3]
        adj <- numeric(nb.tip)
        adj[xx[as.character(1:nb.tip)] < 0] <- 1
        for (i in 1:nb.tip) text(xx[as.character(i)], yy[as.character(i)], 
            x$tip.label[i], font = font, cex = cex, srt = angle[i], 
            adj = adj[i],col=x$col)
    }
    if (show.node.label) 
        text(xx[as.character(-(1:nb.node))] + label.offset, yy[as.character(-(1:nb.node))], 
            x$node.label, adj = adj, font = font, srt = srt, 
            cex = cex)
    L <- list(type = type, use.edge.length = use.edge.length, 
        node.pos = node.pos, show.node.label = show.node.label, 
        edge.color = edge.color, edge.width = edge.width, font = font, 
        cex = cex, adj = adj, srt = srt, no.margin = no.margin, 
        label.offset = label.offset, x.lim = x.lim, y.lim = y.lim, 
        direction = direction)
    .last_plot.phylo <<- c(L, list(xx = xx, yy = yy))
    invisible(L)
}
