
"simARG" <- function(ss,sites,rec,growthmodel="constant"
                     ,migmatrix=NULL,poptree="NULL")
{
  ## void simplesim(int *ss, int *sites, double *rec, int *seed, int *count)

  if (length(rec)==1) r <- rep(rec,sites-1)
  
  r <- 
  .C("simplesim"
     ,as.integer(ss)
     ,as.integer(sites)
     ,as.double(r)
     ,as.character(growthmodel)
     ,as.integer(1)
     )
  r <- list(ss=ss,sites=sites,rec=rec,which=r[[5]]);
  class(r) <- "ARG"
  r
}

"treeheight" <- function(graph)
  {
    .C("treeheight",as.integer(graph$which),as.double(1:graph$sites))[[2]]
  }


 
"treelength" <- function(graph)
  {
    .C("treelength",as.integer(graph$which),as.double(1:graph$sites))[[2]]
  }


"tree" <-  function(graph,position) {
  if (position<1||position>graph$sites) {
    stop("position should be between 1 and sites")
  }
  nchars=8192;
  txt <- .C("extracttree"
            ,as.integer(graph$which)
            ,as.integer(position-1)
            ,as.character(character(nchars))
            ,as.integer(nchars)
            )[[3]];
  
  read.tree(text=txt)
}



"mutate" <- function(graph,var,ascert="panel(3)")
  {
    if (var>0.1*graph$sites||var<1) {
       stop("variable sites must be between 1 and 10% of available sites (but 1% better)")
    }
    ##  void mutate(int *which, int *var, int *data, int *u, char **ascertainment)
    r <- .C("mutate"
            ,as.integer(graph$which)
            ,as.integer(var)
            ,as.integer(numeric(var*graph$ss))
            ,as.integer(numeric(var))
            ,as.character(ascert)
            );
    
    r <- list(ss=graph$ss,sites=graph$sites
              ,rec=graph$rec
              ,d= matrix(r[[3]],ncol=var,byrow=T)
              ,positions=r[[4]]+1
              ,which=graph$which
              ,var=var)
    class(r) <- c("mutatedARG","ARG");
    r;
  }



 "prune" <- function(graph,keep)
  {
    if (length(keep)>graph$ss||length(keep)<2)
       stop("array of samples to keep must have length > 2 and less than sample size")

    r <- range(keep);
    if (r[1]<1||r[2]>graph$ss)
      stop("value to keep must be between 1 and sample size");

    ukeep <- unique(keep)
    if (length(ukeep)<length(keep))
      warning("repeated values in keep array, removed and continuing")
    
    .C("pruneARG"
       ,as.integer(graph$which)
       ,as.integer(keep-1)
       ,as.integer(length(keep))
       )
    graph$keep=ukeep
    graph;
  }


### what are the lengths of haplotypes around position
"haplengths" <- function(graph,position)
  {

    l <- .C("haplengths"
            ,as.integer(graph$which)
            ,as.integer(position-1)
            ,as.double(numeric(2*graph$ss))
            )[[3]]
    matrix(l,nrow=graph$ss)
  }

###
"sharedsection" <- function(graph,position)
  {
    if (is.null(graph$keep))
      wh <- 0:(graph$ss-1)
    else wh <- graph$keep-1;

    print(wh)
    
    l <- .C("sharedsection"
            ,as.integer(graph$which)
            ,as.integer(wh)
            ,as.integer(length(wh))
            ,as.integer(position-1)
            ,as.double(numeric(2*length(wh)))
            )[[5]]+1
    matrix(l,nrow=length(wh))
  }
    
