// @file      Time-stamp: <2005-11-24 11:50:22 nijw>
#ifndef MUTATION_H_IJW__
#define MUTATION_H_IJW__

#include <stdexcept>
// utility
#include "gsl_rand.H"
//Genetics
#include "rnodet.h"

namespace GenTL {
  /** recursively write leaves   */
  template<typename T>
  void write_leaves(rnode<T > *here, position pos)
  {
    if (here->isleaf()) { // at a leaf
      here->data()[pos]=1;
    } else {
      write_leaves(here->left(pos),pos);
      write_leaves(here->right(pos),pos);
    }
    return;
  }
  
  /** Recursively find the position of the mutation              */
  template<typename T>
  double recursemutateINF(rnode<T > *here,double mtime
			  ,const position pos)
  {
    assert(!here->isleaf());
      
    rnode <T > *l(here->left(pos));
    mtime -= here->time-l->time;
    
    if (mtime<0.0) { // mutation happens here
      write_leaves(l,pos);
      return mtime;
    } else { // no mutation here so try to the left
      if (!l->isleaf()) {
	mtime = recursemutateINF(l,mtime,pos);
	if (mtime<0.0) return mtime;
      }
    }
    // no mutation so try to the right
    
    rnode <T > *r(here->right(pos));
    mtime -= here->time-r->time;
    
    if (mtime<0.0) {
      write_leaves(r,pos);
      return mtime;
    } else {
      if (!r->isleaf()) 
	mtime = recursemutateINF(r,mtime,pos);
    }
    return mtime;
  }
  /** First mutation function - infinite site     */
  template<typename T>
  void mutateINF(rnode<T > *root, position pos, rng &r)
  {
    double len=root->length(pos);
    double mut=r()*len;
    
    double mutated=recursemutateINF(root,mut,pos);
    if (mutated>0.0) 
      throw std::domain_error("SNPmutation failed in mutateINF");
  }
}

#endif

