#include "newio.h"
#include <stdexcept>
#include <sstream>

/** read a string from the input stream (ignoring comments) */
vector <string> readstringline(istream &in,int suggestedlength,char until)
{
    vector <string> data;
    data.reserve(suggestedlength);
    string x;
    int ch;
	
    for (;;) {
	ch = skipblank(in,until);
	if ((isdigit(ch))||isalpha(ch)||(ch=='-')||(ch=='+')||(ch=='(')) {
	    in.putback(ch);
	    in >> x;
	    data.push_back(x);
	}
	else break;
    }
    return data;
}
/** read a "matrix" of strings separated by spaces from a file */ 
vector<vector<string> > readstringlines(istream &in)
{
    vector<vector<string> > a;
    // check that this is not a commented out line 
    int ch = skipblank(in,'\n');
    if (ch==EOF) return a;
    in.putback(ch);
    // read in a line
    vector<string> x= readstringline(in);
    int len=x.size();
    a.push_back(x);

    for(;;) {
      int ch = skipblank(in,'\n');
      if (ch==EOF) break;
      in.putback(ch);
      a.push_back(readstringline(in,len));
    } 
    return a;
}
//
//  
//
int skipblank(std::istream &in, char until)
{
  static int ch;
  
  for (;;) {
    ch = in.get();
    if (ch=='#') ch = skipline(in);
    if (ch==until) return (ch);
    if (!isspace(ch)) return (ch);
  }
}
//
//  
//
bool skipto(std::istream &in, const char *pattern, const char comment)
{
  int c;
  const char *p = pattern;

  if (in.eof()) return false;
  while ((c = in.get()) != EOF) {  
    if (*p == c) p++;
    else {
      if (comment == c) skipline(in);
      p = pattern;
    }
    if (!*p) {
      return true;
    }
  }
  return false;
}
//
//
//
int skipline(istream &in)
/*  a little function to read to the end of a line 
    to remove comments !!                            */
{
	char tmp[2000];
	if (in.getline(tmp,2000)==NULL) {
		return EOF;	
	}
	return '\n';
}
//
//
void checkreadcharacter(istream &in ,const char wh, const char *message)
{
   int ch=skipblank(in);
   if (ch!=wh) {
     std::ostringstream oss;
     oss << message << " expected " << wh << ", got " << char(ch);
     throw std::domain_error(oss.str().c_str());
   }     
}
//
//
//

vector <int> readintvector(istream &in,bool usechar, char until)
{
  vector <int> data;
  int x;
 	
  for (;;) {
    int ch = skipblank(in,until);
    //  std::cout << char(ch) << std::endl;
    if ((isdigit(ch))||(ch=='-')||(ch=='N')||(usechar&&isalpha(ch))) {
      if (ch=='N') { // check for NA
	int ch2 = in.get();
	if (ch2=='A') 	x = -1;
	else {
	    if (!usechar)
		throw std::domain_error("got 'N' in readintvector");
	  in.putback(ch2);
	}
      } else {
	in.putback(ch);
	in >> x;
      }			
      data.push_back(x);
    }
    else break;
  }
  return data;
}
//
//
//
vector <vector<int> > readcharintmat(istream &in,size_t &cols, bool usechar)
//  this reads a mixture of integer
{   
    vector<vector<int> > a;
    for (;;) {
      int ch = skipblank(in,'\n');
      if (ch==EOF) break;
      else in.putback(ch);
      vector<int> x= readintvector(in,usechar,'\n');
      a.push_back(x);
    } 
    if (a.size()>0) {
      cols=a[0].size();
      for (size_t i=1;i<a.size();i++) {
	if (a[i].size()!=cols) 
	  throw std::length_error("not a matrix in readcharintmatrix");
      }
    }
    return a;
}


// from http://oopweb.com/CPP/Documents/CPPHOWTO/Volume/C++Programming-HOWTO-7.html
// used as 
// vector<string> tokens;
//    string str("Split me up! Word1 Word2 Word3.");
//    Tokenize(str, tokens);
//
void Tokenize(const string& str,
                      vector<string>& tokens,
                      const string& delimiters)
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}
//
//
//
vector<double> stringtodouble(vector<string> s,size_t from)
{
    if (s.size()<from) std::domain_error("vector opf strings too short to convert");
    vector<double> x;
    x.reserve(s.size()-from);
    char *pEnd;
    for (size_t i=from;i<s.size();i++) {
	x.push_back(strtod(s[i].c_str(),&pEnd));
	if (*pEnd!='\0') {
	    std::ostringstream oss;
	    oss << "problem with string " << s[i] <<" in stringtodouble";
	    throw std::domain_error(oss.str().c_str());
	}
    }
    return x;
}

