// all the functions for R - I shall assume that all functions here 
// do *not* change labels from 0 to n-1 -- rather that shall
// be done in R

#include <deque>
#include <iostream>
#include <string>
#include <vector>
#include <sstream>                // for ostringstream
#include "ARG.H"
#include "nodet.h"                // for node and tree
#include "coalescent.h"
#include "ascertain.h"
#include "growthmodel.h"
#include "migmatrix.h"


//std::string ascertainment="bylength";

//static rng r;

class simrecomb {
public: 
  typedef GenTL::rnode<std::vector<int> > nodetype;
  typedef GenTL::ARG<std::vector<int> ,GenTL::recombinationLocations > ARGtype;

  simrecomb(int ss, int sites,double rec,rng &r):
    g(new GenTL::constant_size())
    ,c(ss,g,r)
    ,tr(sites,std::vector<double>(sites-1,rec),c,r,recomb) {
  };

  simrecomb(int ss, int sites,const std::vector<double> &rec, std::string &gm,rng &r):
     g(gmread(gm))
     ,c(ss,g,r)
     ,tr(sites,rec,c,r,recomb) {
   };

  std::vector<int> mutate(int var,const std::string &a,rng &r);
  //  rng r;
  GenTL::growthmodel *g;
  GenTL::mig_matrix *mm;
  GenTL::tree<double> pt;
  GenTL::coalescent c;
  GenTL::recombinationLocations recomb;
  ARGtype tr;
  bool mutated;
};

std::vector<int>  simrecomb::mutate(int var, const std::string &a,rng &r) 
{
  size_t sites=tr.root.size();
    for (int i=0;i<tr.sample.size();i++) {
    tr.sample[i].data().assign(sites,0);
  } 
   std::vector<double> len(sites);
  for (int i=0;i<sites;i++) len[i]=tr.length(i);
    // now mutate 
  return ascertainedMutation(a,tr,var,len,r);
}



static std::deque<simrecomb *> vsr;

extern "C" {
  void simplesim(int *ss, int *sites, double *rec, char **growthmod, int *count) {
    std::string gm(*growthmod);
    //   std::cerr << gm << std::endl;
    rng r;
    std::vector<double> rr(*sites-1);
    for (int i=0;i<*sites-1;i++) rr[i]=rec[i];
    vsr.push_back(new simrecomb(*ss,*sites,rr,gm,r));
    *count=vsr.size()-1;
  }
  void popsim(int *ss, int *sites, double *rec, char **growthmod, char **migmodel
	      , char **poptree, int *count) {
    rng r;
    std::string gm(*growthmod);
    std::string mm(*migmodel);
    std::string pt(*poptree);
    
    std::cerr << gm <<  " " << mm << " " << pt << std::endl;
    std::vector<double> rr(*sites-1);
    for (int i=0;i<*sites-1;i++) rr[i]=rec[i];
    vsr.push_back(new simrecomb(*ss,*sites,rr,gm,r));
    *count=vsr.size();
  }
  /**  extract the height of the trees in the ARG          */
  void treeheight(int *which, double *times) {
    size_t sites=vsr[*which]->tr.root.size();
    for (int i=0;i<sites;i++) {
      times[i]=vsr[*which]->tr.root[i]->time;
    }
  }
  /** extract the tree lengths from the trees in the ARG   */
  void treelength(int *which, double *times) {
    size_t sites=vsr[*which]->tr.root.size();
    for (int i=0;i<sites;i++) times[i]=vsr[*which]->tr.length(i);
  }
  /** Get the mutations at *var sites in thge ARG          */
  void mutate(int *whichARG, int *var, int *data, int *u, char **ascert) {
    rng r;
    std::string a(*ascert);
    simrecomb::ARGtype &st=vsr[*whichARG]->tr;
    int ss=st.sample.size();
    std::vector<int> use=vsr[*whichARG]->mutate(*var,a,r);
    for (int i=0;i<use.size();i++) u[i]=use[i];
    int count=0;
    for (int i=0;i<ss;i++) 
      for (int j=0;j< *var;j++) 
	data[count++]=st.sample[i].data()[use[j]];
  }
  /** extract the lengths that  prior to the first coalescence                     
      lengths is an array of length 2*ss                                       */
    void haplengths(int *which, int *position, double *lengths) {
      simrecomb::ARGtype &st=vsr[*which]->tr;
      int ss=st.sample.size();
      int sites=st.root.size();
      double *mnp=lengths;
      double *mxp=lengths+ss;
      
      for (int ii=0;ii<ss;ii++) {
	std::set<GenTL::position> pp=st.sample[ii].recombinations();
	pp.insert(-1);
	
	std::set<GenTL::position>::iterator iii=pp.lower_bound(*position);
	iii--;
	if (*iii==-1)  mnp[ii]=1;
	else mnp[ii] = (*iii)+2;
	GenTL::position mp=*(pp.upper_bound(*position));
	if (mp==GenTL::maxloci)  mxp[ii]=sites;
	else mxp[ii] =  mp+1;
      }
      return;
    }
   /** extract the lengths that  prior to the first coalescence                     
       lengths is an array of length 2*ss                                       */
    void sharedsection(int *whichARG, int *samples,int *nsamp, int *position, double *lengths) {
      simrecomb::ARGtype &st=vsr[*whichARG]->tr;
      int ss=st.sample.size();
      int sites=st.root.size();
      std::vector<int> a(samples,samples+(*nsamp));
      
      TNT::Array2D<int> res=st.sharedsections(a,*position);
      int len=res.dim1();
      for (int i=0;i<len;i++) {
	lengths[i]=res[i][0];
	lengths[i+len]=res[i][1];
      }
      return;
    }
    /** extract the tree at a given position                                  */
    void extracttree(int *whichARG, int *position, char **tree, int *nchars) {
      simrecomb::ARGtype &st=vsr[*whichARG]->tr;
      //std::cerr << "extracting tree" << std::endl;
      GenTL::node<simrecomb::nodetype *> 
      	*extractroot=extract_node(st.root[*position]
				  ,*position
				  ,&(st.sample[0]) 
				  );
      //std::cerr << "extracted tree" << std::endl;
      rotate_tree_labels(extractroot);
      std::ostringstream oss;      
      extractroot->recursiveprintnodata(oss,true,false);
      
      std::string os=oss.str();
      if (os.size()>=*nchars) {
	std::cerr << "warning -- need a longer string in extracttree" << std::endl;
	return;
      }
      for (int j=0;j<os.size();j++) *tree[j]=os[j];
      *tree[os.size()]=';';
      recursively_destroy_node(extractroot);
    }
  /** prune an ancestral recombination graph                                  */
    void pruneARG(int *which, int *samples, int *nsamps) {
      simrecomb::ARGtype &st=vsr[*which]->tr;
      std::cerr << "extracting ARG" << std::endl;

      std::vector<size_t> wh(samples,samples + (*nsamps));
      st.prune(wh);
    }
}
